/**
 * LiveServer module
 */
tma.registerModule( liveserver );
function liveserver() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'liveserver';

	var self = this;

	/**
	 * Tap ID
	 */
	this.TapId = 1785416020;

	/**
	 * inject LiveServer Features
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.addApiExtension( this, 'apiStreamFile' );
		tma.addApiExtension( this, 'apiStreamLive' );
		tma.fn.addFeature( this, 'fileCreateMoreFn' );
		tma.fn.addFeature( this, 'channelCreateMoreFn' );

		// load language later (on init)
		return $.Deferred().resolve();

	};

	/**
	 * set file to serve
	 *
	 * @param {String}		dir			path
	 * @param {TmaFile}		file
	 * @returns jqXHR
	 */
	this.apiStreamFile = function (dir, file) {

		var params = {
				'function'	: 'stream',
				action		: 'file',
				tap			: this.TapId + '!',
				dir			: dir,
				file		: file.name || file.filename,
				file_hex	: file.name_hex || file.filename_hex
		};

		/**
		 * this construct allows us to switch to fail(),
		 * if the api returns an error_code <br>
		 * see: return deferred.promise().fail()
		 * http://stackoverflow.com/a/5112734
		 */
		var deferred = $.Deferred();

		$.getJSON('/api', params).done(function(data, status, xhr) {

			if (data.status === 1) {
				deferred.resolve(data, status, xhr);
			} else {
				// switch to fail()
				// TODO think about standardised errors
				deferred.reject(
						xhr,
						'tma-error',
						'LiveServer: ' + data.status_text
				);
			}

		}).fail(function(xhr, errorType, error) {
			deferred.reject(xhr, errorType, error);
		});

		return deferred.promise().fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in liveserver.apiStreamFile()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
					parameter: '',
					directory: dir,
					file: file.name,
					filehex: file.name_hex,
				}
			});

		});

	};

	/**
	 * reset file to serve => stream live TV
	 *
	 * @returns jqXHR
	 */
	this.apiStreamLive = function () {

		var params = {
				'function'	: 'stream',
				action		: 'live',
				tap			: this.TapId + '!',
		};

		return $.getJSON('/api', params).fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in liveserver.apiStreamLive()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
				}
			});

		});

	};

	/**
	 * set channel to serve
	 *
	 * @param {TmaChannel}		channel
	 * @returns jqXHR
	 */
	this.apiStreamChannel = function (channel) {

		var params = {
				'function'	: 'stream',
				action		: 'channel',
				tap			: this.TapId + '!',
				chtype		: channel.chtype,
				chnum		: channel.chnum
		};

		/**
		 * this construct allows us to switch to fail(),
		 * if the api returns an error_code <br>
		 * see: return deferred.promise().fail()
		 * http://stackoverflow.com/a/5112734
		 */
		var deferred = $.Deferred();

		$.getJSON('/api', params).done(function(data, status, xhr) {

			if (data.status === 0) {
				deferred.resolve(data, status, xhr);
			} else {
				// switch to fail()
				// TODO think about standardised errors
				deferred.reject(
						xhr,
						'tma-error',
						'LiveServer: ' + data.status_text
				);
			}

		}).fail(function(xhr, errorType, error) {
			deferred.reject(xhr, errorType, error);
		});

		return deferred.promise().fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in liveserver.apiStreamChannel()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
					parameter: '',
					chtype: channel.chtype,
					chnum: channel.chnum,
				}
			});

		});

	};

	/**
	 * add "stream" button to a files more functions container
	 *
	 * @param {tmaFeature}		f
	 * @param {jQueryObject}	$moreFn		more file functions container
	 * @param {TmaFile}			file
	 */
	this.fileCreateMoreFn = function(f, $moreFn, file) {

		/**
		 * since this contains asynchronous calls, pause the chain
		 * and decide with the result of the call, whether to stop,
		 * or to continue
		 */
		f.pause();

		this.init().done(function() {

			if ( tma.taps.find(self.TapId) ) {

				var $button = $(
						(
						'<button class="jk-but stream" title="{stream_hint}">' +
							'<i class="fa fa-film"></i>' +
							'<span>{stream}</span>' +
						'</button>'
						).format(self.i18.menu)
				).appendTo($moreFn);

				// add event listener to stream file
				$button.on('click', function() {

					// set file to stream
					self.apiStreamFile(tma.files.directory, file)
					.done(function(data) {
						self.showPopup(data.port);
					});

				});

			}

		}).always(function() {

			// continue
			f.run();

		});

	};

	/**
	 * add "stream" button to a channels more functions container
	 *
	 * @param {tmaFeature}		f
	 * @param {jQueryObject}	$moreFn		more channel functions container
	 * @param {TmaChannel}		channel
	 */
	this.channelCreateMoreFn = function(f, $moreFn, channel) {

		/**
		 * since this contains asynchronous calls, pause the chain
		 * and decide with the result of the call, whether to stop,
		 * or to continue
		 */
		f.pause();

		this.init().done(function() {

			if ( tma.taps.find(self.TapId) ) {

				// update status
				tma.apiGetStatus().done(function() {

					var $button = $( (
							'<button class="jk-but stream" title="{stream_hint}">' +
								'<i class="fa fa-film"></i>' +
								'<span>{stream}</span>' +
							'</button>'
					).format(self.i18.menu) );

					if (
							( tma.status.playmode === 0 /*PLAYMODE_None*/ ) &&
							( channel.logoid == tma.status.current_logoid )
					) {

						// current channel => live stream

						$button.appendTo($moreFn)
						// add event listener to stream live
						.on('click', function() {

							// set to live stream
							self.apiStreamLive().done(function(data) {

								self.showPopup(data.port);

							});

						});

					} else {

						// find out, if the requested channel is currently recorded
						var recording = null;
						/** @param {TmaRecording} rec */
						tma.status.eachRecording(function(i, rec) {
							if (rec.logoid == channel.logoid) {
								recording = rec;
								return false; // break;
							}
						}, true);

						if (recording) {

							// channel is a currently recorded channel => set file to serve
							$button.appendTo($moreFn)
							// add event listener to stream file
							.on('click', function() {

								// set file to stream
								self.apiStreamFile( '/mnt/hd/DataFiles', recording )
								.done(function(data) {
									self.showPopup(data.port);
								});

							});


						} else {

							// not current channel and not a recorded channel => set channel to serve

							$button.appendTo($moreFn)
							// add event listener to stream live
							.on('click', function() {

								// set channel to stream
								self.apiStreamChannel(channel).done(function(data) {

									self.showPopup(data.port);

								});

							});

						}

					}

				}).always(function() {

					// continue
					f.run();

				});

			}

		});

	};

	this.showPopup = function(port) {

		var popup = new jkqWindow();
		popup.width = 720;

		popup.show();
		popup.setTitle( 'Livestream' );

		var $content = popup.container.find('DIV.jkqWindowContent');

		var url = ('tcp://{host}:{port}').format( {
			host: window.location.hostname,
			port: port,
		});

		var template =
			'<div class="livestream-box">' +
				'<embed type="application/x-vlc-plugin" ' +
					'name="video" ' +
					'width="720" height="405" ' +
					'autoplay="no" ' +
					'loop="no" hidden="no" ' +
					'target="{url}" ' +
				'/>' +
			'</div>';

		var $template = $( template.format( {url: url} ) )
		.appendTo( $content );

		if (!self.isVLCInstalled()) {
			$template.append(
				('<p class="livestream-novlc">' + self.i18.connect_text + '</p>')
				.format( {url: url} )
			)
			.find('EMBED').hide();
		}

		popup.adjustPos();

	};

	this.isVLCInstalled = function() {

	    if (navigator.plugins && (navigator.plugins.length > 0)) {
	        for(var i=0;i<navigator.plugins.length;++i)
	            if (navigator.plugins[i].name.indexOf('VLC') != -1)
	              return true;
	    }
	    else {
	        try {
	            new ActiveXObject("VideoLAN.VLCPlugin.2");
	            return true;
	        } catch (err) {}
	    }
	    return false;

	};

}
